=== modified file 'CMakeLists.txt'
--- CMakeLists.txt	2015-11-14 17:26:42 +0000
+++ CMakeLists.txt	2016-01-25 23:45:18 +0000
@@ -45,7 +45,7 @@
   add_definitions(-D_POSIX_C_SOURCE=199309L -D_GNU_SOURCE)
   # We need -rdynamic to get the traceback information on *nix.
   # This may not work on all non-windows targets. FIXME.
-  set (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -pthread -std=c11  -Wall -rdynamic -fstack-protector-strong")
+  set (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -pthread -std=c11  -Wall -rdynamic -g -fstack-protector-strong")
 
   set (CMAKE_C_FLAGS_DEBUG "${CMAKE_C_FLAGS_DEBUG} -pthread -std=c11 -Wall -g -rdynamic -fstack-protector-strong")
 

=== modified file 'libssts/LinkedList.c'
--- libssts/LinkedList.c	2016-01-25 21:52:28 +0000
+++ libssts/LinkedList.c	2016-01-25 23:45:18 +0000
@@ -130,14 +130,19 @@
   if (self->len  == 0) { // special case when the list is empty.
     internal_s_linked_list_initialize_first_item (self);
   } else { // general case.
+    SLinkedListNode * old_tail = self->tail;
+    
     self->tail->next       = malloc (sizeof (SLinkedListNode));
     self->tail->next->next = NULL;
     self->tail             = self->tail->next;
+    self->tail->prev       = old_tail;
   }
   self->tail->data = data;
   self->len++;
 
   assert (self->tail->next == NULL);
+  assert (self->tail != NULL);
+  assert (self->current != NULL);
 }
 
 void
@@ -145,14 +150,19 @@
   if (self->len  == 0) { // special case when the list is empty.
     internal_s_linked_list_initialize_first_item (self);
   } else { // general case.
-    self->head->prev = malloc (sizeof (SLinkedListNode));
+    SLinkedListNode * old_head = self->head;
+    
+    self->head->prev       = malloc (sizeof (SLinkedListNode));
     self->head->prev->prev = NULL;
-    self->head = self->head->prev;
+    self->head             = self->head->prev;
+    self->head->next       = old_head;
   }
   self->head->data = data;
   self->len++;
 
   assert (self->head->prev == NULL);
+  assert (self->head != NULL);
+  assert (self->current != NULL);
 }
 
 sboolean
@@ -221,12 +231,18 @@
 void
 s_linked_list_head (SLinkedList * self) {
   self->current = self->head;
+  if (self->len != 0) {
+    assert (self->current != NULL);
+  }
 }
 
 
 void
 s_linked_list_tail (SLinkedList * self) {
   self->current = self->tail;
+  if (self->len != 0) {
+    assert (self->current != NULL);
+  }
 }
 
 
@@ -237,15 +253,16 @@
 
 void
 s_linked_list_remove_current (SLinkedList * self, sboolean free_data) {
+  SLinkedListNode * current = self->current;
+  
+  if (current == NULL) {
+    assert (self->len == 0);
+    s_err_print ("Trying to remove current when there is no items in the list.");
+    return;
+  }
+  
   SLinkedListNode * prev = self->current->prev;
   SLinkedListNode * next = self->current->next;
-  SLinkedListNode * current = self->current;
-
-  
-  if (!s_linked_list_get_current (self)) {
-    s_err_print ("The \"current\" item does not exist. ");
-    return;
-  }
   
   if (free_data) {
     FreeFunc free_func = self->free_func;

=== modified file 'tests/linked_list.c'
--- tests/linked_list.c	2016-01-25 21:52:28 +0000
+++ tests/linked_list.c	2016-01-25 23:45:18 +0000
@@ -24,6 +24,12 @@
   
   sint r_p = rand () % 20;
   
+  s_print (YELLOW
+           "------------------------\n"
+           "APPEND TESTS\n"
+           "------------------------\n"
+           RESET);
+  
   s_linked_list_append (list, (spointer)(intptr_t) r_p);
   
   test_case (s_linked_list_len (list) == 1, "The reported length of the linked"
@@ -44,15 +50,125 @@
   
   s_linked_list_remove_current (list, FALSE);
   
-  s_print ("Item removed");
-  
-  test_case (s_linked_list_len (list) == 0, "The reported list length is 0");
-  
-  s_print ("Filling list with twenty items (0, 1, ... 18, 19)\n");
-  
-  for (int i = 0; i < 20; i++) {
-    s_linked_list_append (list, (spointer)(intptr_t) i);
-  }
+  s_print ("Item removed\n");
+  
+  test_case (s_linked_list_len (list) == 0, "The reported list length is 0");
+  
+  
+  s_print ("Filling list with twenty items (0, 1, ... 18, 19)\n");
+  
+  for (sint i = 0; i < 20; i++) {
+    s_linked_list_append (list, (spointer)(intptr_t) i);
+  }
+  
+  
+  test_case (s_linked_list_len (list) == 20, "The reported length of the list"
+                                             " is 20.");
+  
+  s_print ("Will now print list using a callback.\n");
+  
+  sint itt = 0;
+  
+  s_linked_list_for_each (list, FOREACHFUNC (for_each_print_lst), &itt);
+  
+  s_print (YELLOW "Removing all items, from head.\n" RESET);
+  
+  for (sint i = 0; i < 20; i++) {
+    s_linked_list_head (list);
+    s_linked_list_remove_current (list, FALSE);
+  }
+  
+  test_case (s_linked_list_len (list) == 0, "The reported length of the list"
+                                            " is 0.");
+  
+  s_print ("Filling list with twenty items (19, 18, ... 1, 0)\n");
+  
+  for (sint i = 19; i >= 0; i--) {
+    s_linked_list_append (list, (spointer)(intptr_t) i);
+  }
+  
+  s_print (YELLOW "Removing all items, from tail.\n" RESET);
+  
+  for (sint i = 0; i < 20; i++) {
+    s_linked_list_tail (list);
+    s_linked_list_remove_current (list, FALSE);
+  }
+  
+  test_case (s_linked_list_len (list) == 0, "The reported length of the list"
+                                            " is 0.");
+  
+  s_print (YELLOW
+           "------------------------\n"
+           "PREPEND TESTS\n"
+           "------------------------\n"
+           RESET);
+
+  s_linked_list_prepend (list, (spointer)(intptr_t) r_p);
+  
+  test_case (s_linked_list_len (list) == 1, "The reported length of the linked"
+                                            " list is 1.");
+  
+  test_case (s_linked_list_get_next (list) == NULL, "There are no next items in the"
+                                             " linked list.");
+  
+  test_case (s_linked_list_get_prev (list) == NULL, "There are no previous items in"
+                                             " the linked list.");
+  
+  test_case (s_linked_list_get_current (list) != NULL, "The current item exists.");
+  
+  test_case ((intptr_t) s_linked_list_get_current (list) == r_p, "The value"
+                                 "addad was the same as the one retrieved.");
+  
+  s_print ("Removing current item in preperation for next test.\n");
+  
+  s_linked_list_remove_current (list, FALSE);
+  
+  s_print ("Item removed\n");
+  
+  test_case (s_linked_list_len (list) == 0, "The reported list length is 0");
+  
+  
+  s_print ("Filling list with twenty items (0, 1, ... 18, 19)\n");
+  
+  for (sint i = 0; i < 20; i++) {
+    s_linked_list_prepend (list, (spointer)(intptr_t) i);
+  }
+  
+  
+  test_case (s_linked_list_len (list) == 20, "The reported length of the list"
+                                             " is 20.");
+  
+  s_print ("Will now print list using a callback.\n");
+  
+  itt = 0;
+  
+  s_linked_list_for_each (list, FOREACHFUNC (for_each_print_lst), &itt);
+  
+  s_print (YELLOW "Removing all items, from head.\n" RESET);
+  
+  for (sint i = 0; i < 20; i++) {
+    s_linked_list_head (list);
+    s_linked_list_remove_current (list, FALSE);
+  }
+  
+  test_case (s_linked_list_len (list) == 0, "The reported length of the list"
+                                            " is 0.");
+  
+  s_print ("Filling list with twenty items (19, 18, ... 1, 0)\n");
+  
+  for (sint i = 19; i >= 0; i--) {
+    s_linked_list_prepend (list, (spointer)(intptr_t) i);
+  }
+  
+  s_print (YELLOW "Removing all items, from tail.\n" RESET);
+  
+  for (sint i = 0; i < 20; i++) {
+    s_linked_list_tail (list);
+    s_linked_list_remove_current (list, FALSE);
+  }
+  
+  test_case (s_linked_list_len (list) == 0, "The reported length of the list"
+                                            " is 0.");
   
   s_linked_list_free (list, FALSE);
   

